from django.db import models
from django.db.models.signals import post_delete
from django.dispatch import receiver
from django.utils.translation import gettext_lazy as _

from .core.core import clear_cache


class SmartCache(models.Model):
    key = models.TextField(verbose_name=_('Key'))
    value = models.TextField(verbose_name=_('Value'))
    qs = models.TextField(verbose_name=_('Query String'))
    content_type = models.TextField(verbose_name=_('Content Type'))

    class Meta:
        """Table information."""

        verbose_name = _('Saved cache')
        verbose_name_plural = _('Saved caches')

    def __str__(self):
        return self.key


@receiver(post_delete, sender=SmartCache)
def my_handler(sender, **kwargs):
    clear_cache(kwargs.get('instance').key)



from django.conf import settings

host = getattr(settings, 'SMART_REDIS_HOST', '127.0.0.1')
port = getattr(settings, 'SMART_REDIS_PORT', 6379)
db = getattr(settings, 'SMART_REDIS_DB', 0)
password = getattr(settings, 'SMART_REDIS_PASSWORD', None)
namespace = getattr(settings, 'SMART_REDIS_NAMESPACE', 'cache')
auto_clear_cache = getattr(settings, 'SMART_REDIS_RESTART_AND_CLEAR_CACHE', True)

redis_connect = dict(
    host=host,
    port=port,
    db=db,
    password=password,
    namespace=namespace,
)



import redis
import logging

from django.apps import AppConfig

from .core.core import EzRedis
from .settings import redis_connect, namespace, auto_clear_cache

logger = logging.getLogger(__name__)


class DjangoSmartRedisConfig(AppConfig):
    name = 'django_smart_redis'
    verbose_name = "Smart Cache"

    def ready(self):
        if auto_clear_cache:
            from .models import SmartCache
            
            try:
                with EzRedis(**redis_connect) as r:
                    logger.debug("Scan and delete cache...")
                    for key in r.scan_iter(f"{namespace}:*"):
                        logger.debug(f">>> Delete key '{key.decode()}'")
                        r.delete(key)
                        SmartCache._default_manager.all().delete()

            except redis.exceptions.ConnectionError:
                logger.error("!!! Cache not active !!!")
